//  Projection Mapping Tutorial
//  Windows version
//
//  Receive Spout content from a sender

import spout.*;

Spout spout;
PImage img; // Image to receive a texture
PImage bg0, bg1, bg2, bg3;
int mode; //change Background

void setup() {
  // Initial window size
  size(640, 480, P3D);
  
  // Create an image to receive the data.
  // MAPIO Spout Output has alpha information
  img = createImage(width, height, ARGB);
  
  // Create a new Spout object
  spout = new Spout(this);
  
  mode = 1;
  bg0 =loadImage("mannequin_00.jpg");
  bg1 =loadImage("mannequin_01.jpg");
  bg2 =loadImage("statue_00.jpg");
  bg3 =loadImage("statue_01.jpg");
} 

void draw() {
  background(0);

  //Change bg image
  switch(mode) {
    case 1: 
      image(bg0, 0, 0);
      break;
    case 2: 
      image(bg1, 0, 0);
      break;
    case 3: 
      image(bg2, 0, 0);
      break;
    case 4: 
      image(bg3, 0, 0);
      break;
  }

  // We need to do a bit of image processing on the received
  // Spout image, so we store it into PImage pixels
  img = spout.receivePixels(img);

  // We are going to access Spout image's pixels...
  img.loadPixels(); 
  // ...and screen bg image pixels
  loadPixels(); 
  
  // Cycle through each image pixel
  for (int y = 0; y < height; y++) {
    for (int x = 0; x < width; x++) {
      int loc = x + y*width;
      
      // The functions red(), green(), and blue()
      // pull out the 3 color components from a pixel.
      
      // Read the Spout image current pixel value
      float r = red(img.pixels[loc]);
      float g = green(img.pixels[loc]);
      float b = blue(img.pixels[loc]);
      float a = alpha(img.pixels[loc]);
      
      // Read the bg image current pixel value
      float r_bg = red(pixels[loc]);
      float g_bg = green(pixels[loc]);
      float b_bg = blue(pixels[loc]);

      // Image Processing: Mutiply blend mode between
      // the Spout image and the bg image
      float r_new = r_bg * (r/255);
      float g_new = g_bg * (g/255);
      float b_new = b_bg * (b/255);
      img.pixels[loc] =  color(r_new, g_new, b_new, a); 
    }
  }

  // Update the Spout image with the calculated Multiply
  // blending mode, and display it
  img.updatePixels();
  image(img, 0, 0, width, height);
}

// SELECT A SPOUT SENDER
void mousePressed() {
  // RH click to select a sender
  if (mouseButton == RIGHT) {
    // Bring up a dialog to select a sender.
    // Spout installation required
    spout.selectSender();
  }
}

void keyPressed() {
  // Change bg img
  if (key == '1')
    mode = 1;
  else if  (key == '2')
    mode = 2;
  else if  (key == '3')
    mode = 3;
  else if  (key == '4')
    mode = 4;
}
      